@@ -11,8 +11,9 @@ use vars qw($DefaultPriority @ATTRIBUTE);
 sub new {
 #    lock %Event::;
 
-    my $o = allocate(shift);
+    my $class = shift;
     my %arg = @_;
+    my $o = allocate($class, delete $arg{attach_to} || {});
     
     # deprecated
     for (qw(min max repeat)) {
