@@ -9,8 +9,9 @@ use vars qw(@ATTRIBUTE);
 sub new {
     # lock %Event::;
 
-    my $o = allocate(shift);
+    my $class = shift;
     my %arg = @_;
+    my $o = allocate($class, delete $arg{attach_to} || {});
     $o->init(\%arg);
     $o->start;
     $o;
